declare module '@ember/routing/lib/utils' {
  import type { ControllerQueryParam, ControllerQueryParamType } from '@ember/controller';
  import type { ModelFor } from 'router_js';
  import type Router from 'router_js';
  import type { ExtendedInternalRouteInfo } from '@ember/routing/route';
  import type Route from '@ember/routing/route';
  import type EmberRouter from '@ember/routing/router';
  export type ExpandedControllerQueryParam = {
    as: string | null;
    scope: string;
    type?: ControllerQueryParamType;
  };
  export type NamedRouteArgs<R extends Route> =
    | [routeNameOrUrl: string, ...modelsAndOptions: [...ModelFor<R>[], RouteOptions]]
    | [routeNameOrUrl: string, ...models: ModelFor<R>[]];
  export type UnnamedRouteArgs<R extends Route> =
    | [...modelsAndOptions: [...ModelFor<R>[], RouteOptions]]
    | [...models: ModelFor<R>[]]
    | [options: RouteOptions];
  export type RouteArgs<R extends Route> = NamedRouteArgs<R> | UnnamedRouteArgs<R>;
  type ExtractedArgs<R extends Route> = {
    routeName: string | undefined;
    models: ModelFor<R>[];
    queryParams: Record<string, unknown>;
  };
  export type RouteOptions = {
    queryParams: Record<string, unknown>;
  };
  export function extractRouteArgs<R extends Route>(args: RouteArgs<R>): ExtractedArgs<R>;
  export function getActiveTargetName(router: Router<Route>): string;
  export function stashParamNames<R extends Route>(
    router: EmberRouter<R>,
    routeInfos: Array<ExtendedInternalRouteInfo<R>> & {
      _namesStashed?: boolean;
    }
  ): void;
  export function calculateCacheKey(
    prefix: string,
    parts: string[] | undefined,
    values: {} | null
  ): string;
  export function normalizeControllerQueryParams(
    queryParams: Readonly<ControllerQueryParam[]>
  ): Record<string, ExpandedControllerQueryParam>;
  export function resemblesURL(str: unknown): str is string;
  export function prefixRouteNameArg<T extends NamedRouteArgs<Route> | UnnamedRouteArgs<Route>>(
    route: Route,
    args: T
  ): T;
  export function shallowEqual<A extends object, B extends object>(a: A, b: B): boolean;
  export function deprecateTransitionMethods(frameworkClass: string, methodName: string): void;
  export {};
}
